<?php

// Test suite for fxe.php

require __DIR__.'/fxe.php';

function fxtest($sz, $expect) {
	$res = FXEngine::exec($sz);
	echo $sz." = ".var_export($res, true)." ".($res === $expect ? '(passed)' : 'FAILED')."\n";
}

fxtest('(3 + 5)', 8);
fxtest('3 + 5', 8);
fxtest('2 + -1', 1);
fxtest('3 * 4', 12);
fxtest('4 / 2', 2);
fxtest('3 / 4', 0.75);
fxtest('3 / 0', 0);
fxtest('1', 1);
fxtest('1 + 1 + 1', 3);
fxtest('1 + 2 * 3', 7);
fxtest('1 + (2 * 3)', 7);
fxtest('(1 + 2) * 3', 9);
fxtest('5 = 5', 1);
fxtest('5 = 6', 0);
fxtest('5 > 6', 0);
fxtest('6 < 7', 1);

// Test errors

fxtest('', 'Unexpected number of nested elements');
fxtest('1)(1', 'Brace mismatch');
fxtest('+', 'Unexpected element (expected number)');
fxtest('1 + + 1', 'Unexpected number of elements');
fxtest('1 - - 1', 'Unexpected number of elements');
fxtest('1 - -1', 2);
fxtest('- 1', 'Unexpected number of elements');
fxtest('-1', -1);
fxtest('(((2*(1', 'Brace mismatch');
fxtest(')))2*)1', 'Brace mismatch');
fxtest('1,2', 'Unexpected number of nested elements');
fxtest('3 $ 4', 'Unexpected token in source near "$ 4)"');

// functions

fxtest('if(4 > 2, 100, 200)', 100);
fxtest('if(4 < 2, 100, 200)', 200);
fxtest('if(1, 100)', 100);
fxtest('if(0, 100)', 0);

fxtest('min(3, 4, 5, 2)', 2);
fxtest('max(3, 4, 5, 2)', 5);
fxtest('min(1)', 1);
fxtest('max(1)', 1);

fxtest('1 + and(1, 2, 3)', 2);
fxtest('1 + and(1, 2 > 5)', 1);
fxtest('1 + or(1, 2, 3)', 2);
fxtest('1 + or(1, 2 > 5)', 2);
fxtest('1 + and()', 'Unexpected number of arguments to and()');
